﻿///////////////////////////////////////////////////////////////////////////////
// Activity: program.activity0
// ForLoop service implementation
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml;

using ccr = Microsoft.Ccr.Core;
using dss = Microsoft.Dss.Core;
using dssa = Microsoft.Dss.Core.Attributes;
using dssh = Microsoft.Dss.Core.DsspHttp;
using dssm = Microsoft.Dss.ServiceModel.DsspServiceBase;
using dssp = Microsoft.Dss.ServiceModel.Dssp;
using soap = W3C.Soap;

using submgr = Microsoft.Dss.Services.SubscriptionManager;

namespace ProMRDS.Customactivityorder.ForLoop
{
    [DisplayName("Customactivityorder ForLoop")]
    [Description("Implements a For Loop")]
    [dssa.Contract(Contract.Identifier)]
    public class ForLoopService : dssm.DsspServiceBase
    {
        // Service state
        [dssa.InitialStatePartner(Optional = true)]
        private ForLoopState _state;

        // Service operations port
        [dssa.ServicePort("/Customactivityorder/ForLoop", AllowMultipleInstances = true)]
        private ForLoopOperations _mainPort = new ForLoopOperations();

        #region Partner services

        [dssa.Partner("SubMgr", Contract = submgr.Contract.Identifier, CreationPolicy = dssa.PartnerCreationPolicy.CreateAlways)]
        private submgr.SubscriptionManagerPort _subMgr = new submgr.SubscriptionManagerPort();

        ForLoop.ForLoopOperations _forLoopPort = new ForLoop.ForLoopOperations();

        #endregion

        public ForLoopService(dssp.DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
        }

        protected override void Start()
        {
            // If there was no initial state partner, then the service state will be null.
            if (_state == null)
            {
                // The state MUST be created before the service starts processing messages.
                _state = new ForLoopState();
            }

            // establish forwarder to this service
            _forLoopPort = base.ServiceForwarder<ForLoopOperations>(ServiceInfo.Service);

            // The rest of the start process requires the ability to wait for responses from
            // services and from the start handler (if any). So execution now proceeds in an
            // iterator function.
            SpawnIterator(DoStart);
        }

        private IEnumerator<ccr.ITask> DoStart()
        {

            // Start operation handlers and insert into directory service.
            StartHandlers();

            yield break;
        }

        private void StartHandlers()
        {
            // Activate message handlers for this service and insert into the directory.
            base.Start();
        }

        #region Standard DSS message handlers

        [dssa.ServiceHandler(dssa.ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ccr.ITask> GetHandler(Get get)
        {
            get.ResponsePort.Post(_state);
            yield break;
        }

        [dssa.ServiceHandler(dssa.ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ccr.ITask> HttpGetHandler(dssh.HttpGet httpGet)
        {
            httpGet.ResponsePort.Post(new dssh.HttpResponseType(_state));
            yield break;
        }

        [dssa.ServiceHandler(dssa.ServiceHandlerBehavior.Exclusive)]
        public virtual IEnumerator<ccr.ITask> ReplaceHandler(Replace replace)
        {
            _state = replace.Body;

            replace.ResponsePort.Post(dssp.DefaultReplaceResponseType.Instance);
            base.SendNotification<Replace>(_subMgr, replace);

            yield break;
        }

        [dssa.ServiceHandler(dssa.ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ccr.ITask> SubscribeHandler(Subscribe subscribe)
        {
            dssp.SubscribeRequestType request = subscribe.Body;

            yield return ccr.Arbiter.Choice(
                SubscribeHelper(_subMgr, request, subscribe.ResponsePort),
                delegate(ccr.SuccessResult success)
                {
                    base.SendNotificationToTarget<Replace>(request.Subscriber, _subMgr, _state);
                },
                delegate(Exception e) { }
            );
        }

        #endregion

        #region Custom message handlers

        [dssa.ServiceHandler(dssa.ServiceHandlerBehavior.Exclusive)]
        public virtual IEnumerator<ccr.ITask> InitializeHandler(Initialize message)
        {
            InitializeMessageHandler handler = new InitializeMessageHandler(this, Environment.TaskQueue);

            SpawnIterator(message.Body, message.ResponsePort, handler.RunHandler);

            yield return ccr.Arbiter.Receive(false, handler.Complete, EmptyHandler);

            if (handler.StateChanged == true)
            {
                base.SendNotification<Replace>(_subMgr, _state);
            }
        }

        [dssa.ServiceHandler(dssa.ServiceHandlerBehavior.Exclusive)]
        public virtual IEnumerator<ccr.ITask> CountHandler(Count message)
        {
            CountMessageHandler handler = new CountMessageHandler(this, Environment.TaskQueue);

            SpawnIterator(message.Body, message.ResponsePort, handler.RunHandler);

            yield return ccr.Arbiter.Receive(false, handler.Complete, EmptyHandler);

            if (handler.StateChanged == true)
            {
                base.SendNotification<Replace>(_subMgr, _state);
            }
        }

        #endregion

        #region InitializeMessageHandler class

        class InitializeMessageHandler : HandlerBase
        {
            ///////////////////////////////////////////////////////////////////
            // request - [(expr0 - set0 - join)(expr - set - &join)]
            // snippet.snippet.noop - [(snippet.snippet.expr0 - snippet.snippet.join)(snippet.snippet.expr - &snippet.snippet.join)] - snippet.element
            // __use__0.snippet0.call - __use__0.snippet0.call.iftype
            ///////////////////////////////////////////////////////////////////

            public InitializeMessageHandler(ForLoopService service, ccr.DispatcherQueue queue)
                : base(service, queue)
            {
            }

            dssp.DsspResponsePort<InitializeResponse> _responsePort;

            ///////////////////////////////////////////////////////////////////
            // request - [(expr0 - set0 - join)(expr - set - &join)]
            // __use__0.snippet0.call - __use__0.snippet0.call.iftype
            // snippet.snippet.noop - [(snippet.snippet.expr0 - snippet.snippet.join)(snippet.snippet.expr - &snippet.snippet.join)] - snippet.element
            ///////////////////////////////////////////////////////////////////

            public IEnumerator<ccr.ITask> RunHandler(InitializeRequest message, dssp.DsspResponsePort<InitializeResponse> responsePort)
            {
                _responsePort = responsePort;
                Increment();

                base.StateChanged = true;

                ForLoopState a = new ForLoopState();
                State.End = message.EndValue;
                a.End = State.End;
                base.StateChanged = true;

                ForLoopState b = new ForLoopState();
                State.Current = message.StartValue;
                b.Current = State.Current;
                JoinAlpha c = new JoinAlpha();
                c.EndValue = a;
                c.StartValue = b;

                ForLoopPort.Count(new ForLoop.CountRequest());

                InitializeResponse response = new InitializeResponse();
                _responsePort.Post(response);

                Decrement();

                yield return WaitUntilComplete();
            }

            class JoinAlpha
            {
                public ForLoopState StartValue;
                public ForLoopState EndValue;

                public JoinAlpha()
                {
                }

                public JoinAlpha(object[] args)
                {
                    StartValue = (ForLoopState)args[0];
                    EndValue = (ForLoopState)args[1];
                }
            }

            ccr.Port<object>[] _joinAlphaPorts = new ccr.Port<object>[2]{
                new ccr.Port<object>(),
                new ccr.Port<object>()
            };

            class JoinBeta
            {
                public string StartValue;
                public string EndValue;

                public JoinBeta()
                {
                }

                public JoinBeta(object[] args)
                {
                    StartValue = args[0].ToString();
                    EndValue = args[1].ToString();
                }
            }

            ccr.Port<object>[] _joinBetaPorts = new ccr.Port<object>[2]{
                new ccr.Port<object>(),
                new ccr.Port<object>()
            };
        }

        #endregion

        #region CountMessageHandler class

        class CountMessageHandler : HandlerBase
        {
            ///////////////////////////////////////////////////////////////////
            // if if{expr2 - set1 - expr3}else{}
            // [(snippet0.snippet.noop - snippet0.snippet.expr - snippet0.snippet.join)] - snippet0.element
            // snippet.element
            ///////////////////////////////////////////////////////////////////

            public CountMessageHandler(ForLoopService service, ccr.DispatcherQueue queue)
                : base(service, queue)
            {
            }

            dssp.DsspResponsePort<CountResponse> _responsePort;

            ///////////////////////////////////////////////////////////////////
            // if if{expr2 - set1 - expr3}else{}
            // snippet.element
            // [(snippet0.snippet.noop - snippet0.snippet.expr - snippet0.snippet.join)] - snippet0.element
            ///////////////////////////////////////////////////////////////////

            public IEnumerator<ccr.ITask> RunHandler(CountRequest message, dssp.DsspResponsePort<CountResponse> responsePort)
            {
                _responsePort = responsePort;
                Increment();

                if (State.Current <= State.End)
                {
                    base.StateChanged = true;

                    ForLoopState a = new ForLoopState();
                    State.Current = State.Current + 1;
                    a.Current = State.Current;

                    _responsePort.Post(new CountResponse());

                    int b = State.Current - 1;
                    CountNotificationBody notify = new CountNotificationBody();
                    notify.Count = b;
                    SendNotification<CountNotification>(notify);
                }

                Decrement();

                yield return WaitUntilComplete();
            }

            class JoinAlpha
            {
                public string Count;

                public JoinAlpha()
                {
                }

                public JoinAlpha(object[] args)
                {
                    Count = args[0].ToString();
                }
            }

            ccr.Port<object>[] _joinAlphaPorts = new ccr.Port<object>[1]{
                new ccr.Port<object>()
            };
        }

        #endregion

        #region Handler utility base class

        class HandlerBase : ccr.CcrServiceBase
        {
            ccr.Port<ccr.EmptyValue> _complete = new ccr.Port<ccr.EmptyValue>();
            ccr.Port<ccr.EmptyValue> _shutdown = new ccr.Port<ccr.EmptyValue>();
            List<ccr.Port<object>[]> _joins = new List<ccr.Port<object>[]>();
            ForLoopService _service;
            int _count;

            public HandlerBase(ForLoopService service, ccr.DispatcherQueue queue)
                : base(queue)
            {
                _service = service;
            }

            protected ForLoopState State
            {
                get { return _service._state; }
            }

            bool _stateChanged;

            public bool StateChanged
            {
                get { return _stateChanged; }
                set { _stateChanged = value; }
            }

            protected ForLoop.ForLoopOperations ForLoopPort
            {
                get { return _service._forLoopPort; }
            }

            public ccr.Port<ccr.EmptyValue> Complete
            {
                get { return _complete; }
            }

            protected void Increment()
            {
                System.Threading.Interlocked.Increment(ref _count);
            }

            protected void Decrement()
            {
                int offset = 0;

                foreach (ccr.Port<object>[] join in _joins)
                {
                    bool complete = true;
                    int joinCount = 0;

                    foreach (ccr.Port<object> sink in join)
                    {
                        if (sink.ItemCount != 0)
                        {
                            joinCount += sink.ItemCount;
                        }
                        else
                        {
                            complete = false;
                        }
                    }
                    if (complete == false)
                    {
                        offset += joinCount;
                    }
                }

                if (System.Threading.Interlocked.Decrement(ref _count) <= offset)
                {
                    _shutdown.Post(ccr.EmptyValue.SharedInstance);
                    _complete.Post(ccr.EmptyValue.SharedInstance);
                }
            }

            protected void Decrement(int count)
            {
                for(int i = 0; i < count; i++)
                {
                    Decrement();
                }
            }

            protected void RegisterJoin(ccr.Port<object>[] join)
            {
                _joins.Add(join);
            }

            protected ccr.ITask WaitUntilComplete()
            {
                return ccr.Arbiter.Receive(false, _shutdown, NullDelegate);
            }

            protected static void NullDelegate(ccr.EmptyValue token)
            {
            }

            protected void SendNotification<T>(object notification)
                where T : dssp.DsspOperation, new()
            {
                _service.SendNotification<T>(_service._subMgr, notification);
            }

            protected void FaultHandler(soap.Fault fault, string msg)
            {
                _service.LogError(null, msg, fault);
            }

            protected static string Stringize(object obj)
            {
                if (obj == null)
                {
                    return string.Empty;
                }
                else
                {
                    return obj.ToString();
                }
            }

            protected void UnhandledResponse(W3C.Soap.Fault fault)
            {
                _service.LogError("Unhandled fault response from partner service", fault);
                Decrement();
            }

            protected void UnhandledResponse<T>(T response)
            {
                Decrement();
            }

            #region Type Cast Functions

            protected static bool BoolCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToBoolean(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return false;
            }

            protected static byte ByteCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToByte(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static char CharCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToChar(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return '\0';
            }

            protected static decimal DecimalCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToDecimal(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static double DoubleCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToDouble(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static float FloatCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToSingle(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static int IntCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToInt32(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static long LongCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToInt64(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static sbyte SByteCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToSByte(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static short ShortCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToInt16(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static uint UIntCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToUInt32(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static ulong ULongCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToUInt64(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            protected static ushort UShortCast(object obj)
            {
                if (obj != null && obj is IConvertible)
                {
                    try
                    {
                        return ((IConvertible)obj).ToUInt16(System.Globalization.CultureInfo.InvariantCulture);
                    }
                    catch (FormatException)
                    {
                    }
                }
                return 0;
            }

            #endregion

            #region List Functions

            protected List<T> ListAdd<T>(List<T> list, T item)
            {
                List<T> output = new List<T>(list);
                output.Add(item);
                return output;
            }

            protected List<T> ListConcat<T>(List<T> head, List<T> tail)
            {
                List<T> output = new List<T>(head);
                output.AddRange(tail);
                return output;
            }

            protected int ListIndex<T>(List<T> list, T item)
            {
                return list.IndexOf(item);
            }

            protected List<T> ListRemove<T>(List<T> list, int index)
            {
                List<T> output = new List<T>(list);
                output.RemoveAt(index);
                return output;
            }

            protected List<T> ListReverse<T>(List<T> list)
            {
                List<T> output = new List<T>(list);
                output.Reverse();
                return output;
            }

            protected List<T> ListSort<T>(List<T> list)
            {
                List<T> output = new List<T>(list);
                output.Sort();
                return output;
            }

            protected List<T> ListInsert<T>(List<T> list, T item, int index)
            {
                List<T> output = new List<T>(list);
                output.Insert(index, item);
                return output;
            }

            #endregion
        }

        #endregion
    }
}
