//------------------------------------------------------------------------------
//
//  Integrator Drive Types
//
//  Written Jan-2008 by Trevor Taylor, Software Technology, Australia
//
//------------------------------------------------------------------------------

using Microsoft.Dss.Core.Attributes;
using System;
using System.Collections.Generic;

// Needed for attributes
using System.ComponentModel;
// For port stuff
using Microsoft.Ccr.Core;
// For Fault
using W3C.Soap;
// For standard Dssp types
using Microsoft.Dss.ServiceModel.Dssp;


namespace ProMRDS.Robotics.Integrator.Drive
{
    /// <summary>
    /// IntegratorDrive Contract
    /// </summary>
    public sealed class Contract
    {   
        /// The Unique Contract Identifier for the IntegratorDrive service
        public const String Identifier = "http://www.promrds.com/contracts/2008/01/integratordrive.html";
    }

    // An additional (internal) operation
    [DisplayName("Stop Timed Motion")]
    [Description("Stops a specified motion request (if still active)")]
    public class StopMotion : Submit<StopMotionRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
        public StopMotion()
        {
        }
        public StopMotion(StopMotionRequest stop)
        {
            this.Body = stop;
        }
        public StopMotion(int num)
        {
            this.Body.AckNumber = num;
        }
    }

    [Description("Class to terminate a timed move -- Internal")]
    //    [DataContract]
    //    [DataMemberConstructor]
    public class StopMotionRequest
    {
        private int _num;

        [DataMember]
        [Description("Acknowledgement number")]
        public int AckNumber
        {
            get { return this._num; }
            set { this._num = value; }
        }
    }

}
